# Copyright 2022 Thinh Vu @ GitHub
# See LICENSE for details.

__author__ = "Thinh Vu @thinh-vu in GitHub"
__version__ = "0.0.2"

from .util import *

from .datetime_intel import (
    date_from_today,
    date_start_end,
    weeknum_intel,
    month_intel,
    quarter_intel
)

from .database_connect import (
    db_config,
    db_query
)

from .messaging_api import (
    lmt_detect,
    slack_send_file,
    slack_send_message,
    telegram_send_message,
    telegram_send_photo
)

from .google_service import(
    google_auth,
    read_gsheet,
    save_gsheet,
    save_gspread
)

from .ai_ml import (
    transcribe,
    video_title,
    free_memory
)

from .visualization import (
    wordcloud_gen,
    word_tokenize_stats
)


# Generate word cloud
from wordcloud import WordCloud
from .util import get_google_font

# Words tokenize
import nltk
nltk.download('punkt')
from nltk.tokenize import word_tokenize
from collections import Counter

# Visualization with Matplotlib
import pandas as pd
import matplotlib.pyplot as plt
#%matplotlib inline

def wordcloud_gen (text, font_path='', limit=100, color_map='magma', bg_color='white', font_size=300, fig_size= (20,20), width=1024, height=800, interpolation='bilinear'):
    if font_path == '':
        custom_font = ''
    else:
        custom_font = font_path

    wordcloud = WordCloud(background_color=bg_color, 
                        max_words=limit,
                        max_font_size=font_size,
                        width=width, 
                        height=height,
                        colormap=color_map,
                        font_path = custom_font
                        ).generate(text)

    plt.figure(figsize=fig_size)
    plt.imshow(wordcloud, interpolation=interpolation)
    plt.axis("off")
    plt.margins(x=0, y=0)
    return plt.show()

def word_tokenize_stats(text):
    c = Counter(word_tokenize(text))
    df = pd.DataFrame.from_records(list(dict(c).items()), columns=['word','count']).sort_values(by='count', ascending=False).reset_index(drop=True)
    return df


# Copyright 2022 Thinh Vu @ GitHub
# See LICENSE for details.
import pandas as pd
import requests
import json
from pandas import json_normalize
import urllib.request, shutil #, zipfile
import time

# HUBSPOT
# contact_id = hs_object_id
def hs_search_contact(user_email, token_key, properties_list):
  """Search contact properties as a DataFrame with an email.
  Args:
        user_email (:obj:`str`, required): target user email to search.
        token_key (:obj:`str`, required): HubSpot Private App Token. See more: https://developers.hubspot.com/docs/api/private-apps
        properties_list (:obj:`list`, required): A list of properties you need to retrieve information. Ex: ['email', 'createdate', 'lastmodifieddate']
  """
  url = 'https://api.hubapi.com/crm/v3/objects/contacts/search'
  payload = json.dumps({
    "filterGroups": [
      {
        "filters": [
          {
            "value": "{}".format(user_email),
            "propertyName": "email",
            "operator": "EQ"
          }
        ]
      }
    ],
    "properties": "{}".format(properties_list)
  })
  headers = {'Authorization' : 'Bearer {}'.format(token_key),
             'content-type': 'application/json'}
  response = requests.post(url, headers=headers, data=payload).json()
  result = json_normalize(response['results'])
  result.columns = result.columns.str.replace('properties.','')
  return result

# FRESHCHAT

def fc_report_read(report_content):
  """Convert JSON content into DataFrame using the output of the fc_retrieve_report function.
  Args:
  -----
  report_content (:obj:`str`, required): json contetn returned from the fc_retrieve_report function
  """
  for i in range(len(report_content['links'])):
    download_link = report_content['links'][i]['link']['href']
    file_name = 'freshchat_report2.zip'
    with urllib.request.urlopen(download_link) as response, open(file_name, 'wb') as out_file:
        shutil.copyfileobj(response, out_file)
    if i == 0:
      fc_report_df = pd.read_csv(file_name)
    elif i > 0:
      fc_report_df1 = pd.read_csv(file_name)
      fc_report_df = pd.concat([fc_report_df, fc_report_df1]).reset_index(drop=True)
  return fc_report_df


def fc_retrieve_report(report_id, token_key): 
  """Retrieve a report by report ID
  Args:
  -----
  report_id (:obj:`str`, required): a report_id that generated by the fc_extract_report function.
  token_key (:obj:`str`, required): FreshChat token key. For an approved Freshchat app, navigate to the `Settings` > `API Tokens` page and click the `Generate Token` button.
  """
  url = 'https://api.freshchat.com/v2/reports/raw/{}'.format(report_id)
  payload = {}
  headers = {'Authorization' : 'Bearer {}'.format(token_key)}
  response = requests.request("GET", url, headers=headers, data=payload, stream=True).json()
  return response


def fc_report_request(start_date, end_date, report_type, token_key): # Conversation-Created 
  """Schedule to extract a report by report type and generate report id
  Args:
  -----
  start_date (:obj:`str`, required): Report start date in the `%Y-%m-%d` format. Ex: `2022-09-01`
  end_date (:obj:`str`, required): Report the end date in the `%Y-%m-%d` format. The reported range should be within 30 days. Ex: `2022-09-01`.
  report_type (:obj:`str`, required): Choose one of following report type: Conversation-Created, Chat-Transcript, Message-Sent, Conversation-Resolved, Conversation-Resolution-Label, Conversation-Activity, etc. See the full list here: https://developers.freshchat.com/api/#reports
  token_key (:obj:`str`, required): FreshChat token key. For an approved Freshchat app, navigate to the `Settings` > `API Tokens` page and click the `Generate Token` button.
  """
  url = 'https://api.freshchat.com/v2/reports/raw'
  payload = json.dumps({"start": "{}".format(start_date), 
            "end": "{}".format(end_date), 
            "event": "{}".format(report_type), 
            "format": "csv"
            })
  headers = {'Authorization' : 'Bearer {}'.format(token_key), 'Accept': 'application/json', 'Content-Type': 'application/json'}
  response = requests.request("POST", url, headers=headers, data=payload).json()
  report_id = response['id']
  return report_id




import requests
import json
import os
from .util import lmt_detect

# SLACK API
def slack_send_file(token_key, slack_channel, text_comment, file_path, title=None):
  """
  Send a file to a Slack channel using either a bot or a user token
  Args:
      token_key (:obj:`str`, required): a bot token (start with 'xoxb-..') or a user token (start with 'xoxp-..')
      slack_channel (:obj:`str`, required): name of the target channel. Ex: '#mkt_daily_tracking'
      text_comment (:obj:`str`, required): The text comment for your file.
      file_path (:obj:`str`, required): The path to your target file
  """
  file_name = file_path.split(lmt_detect())[-1]
  file_type = file_name.split('.')[-1]
  file_bytes = open(file_path,'rb').read()
  url = 'https://slack.com/api/files.upload'
  payload =  {'token': token_key,
              'filename': file_name,
              'channels': slack_channel,
            'fi1letype': file_type,
              'initial_comment': text_comment,
              'title': title
              }
  r = requests.post(url, payload, files={ 'file': file_bytes })
  return r.json()


def slack_send_message (token_key, slack_channel, message):
  """
  Send a file to a Slack channel using either a bot or a user token
  Args:
      token_key (:obj:`str`, required): a bot token (start with 'xoxb-..') or a user token (start with 'xoxp-..')
      slack_channel (:obj:`str`, required): name of the target channel. Ex: '#mkt_daily_tracking'
      message (:obj:`str`, required): The text message for your file.
  """
  header = {'Content-type': 'application/json; charset=utf-8',
          'Authorization': 'Bearer {}'.format(token_key)
          }
  payload = json.dumps({
    "channel": "{}".format(slack_channel),
    "text": "{}".format(message)
    })
  r = requests.post('https://slack.com/api/chat.postMessage',
      data=payload, headers = header)
  return r.json()

# TELEGRAM API
def telegram_send_photo(token_key, chat_id, message, file_path):
    """
    Send a file to a Telegram group.
    Args:
        token_key (:obj:`str`, required): telegram token key
        chat_id (:obj:`str`, required): id of the target telegram channel/group. Ex: '-1001439492355'
        message (:obj:`str`, required): Your text message.
        file_path (:obj:`str`, required): path of your file/photo to send via telegram.
    """
    file_name = file_path.split(lmt_detect())[-1]
    file_type = file_name.split('.')[-1]  
    files=[('photo',(file_name, open(file_path,'rb'),'image/{}'.format(file_type)))]
    url = 'https://api.telegram.org/{}/sendPhoto'.format(token_key)
    payload = {'chat_id': chat_id,'caption': message}
    headers = {}
    response = requests.request("POST", url, headers=headers, data=payload, files=files)
    return response

def telegram_send_message(token_key, chat_id, message):
    """
    Send a message to a Telegram group.
    Args:
        token_key (:obj:`str`, required): telegram token key
        chat_id (:obj:`str`, required): id of the target telegram channel/group. Ex: '-1001439492355'
        message (:obj:`str`, required): Your text message.
    """
    tel_url = 'https://api.telegram.org/{}/sendMessage?chat_id={}&text={}'.format(token_key, chat_id, message)
    return requests.post(tel_url)


# ## Simple function to get an object's name as a string
# def name_of_global_obj(target_object):
#     return [objname for objname, oid in globals().items()
#             if id(oid)==id(target_object)][0]

# # Upadate contact properties to HubSpot
# def hs_update_contact(hs_contact_id, **property_name_also_an_object, token_key):
#   """
#   Update contact properties by a specific contact id which can be obtained by a search API call. 
#   Args:
#         hs_contact_id (:obj:`str`, required): hubspot id of a specific contact/email.
#         token_key (:obj:`str`, required): HubSpot Private App Token. See more: https://developers.hubspot.com/docs/api/private-apps
#         properties_list (:obj:`list`, required): A list of properties you need to retrieve information. Ex: ['email', 'createdate', 'lastmodifieddate']
#   """
# #   prop_1 = name_of_global_obj(property_name_also_an_object)
#   url = 'https://api.hubapi.com/crm/v3/objects/contacts/{}'.format(hs_contact_id)
#   payload = json.dumps({
#   "properties": {
#     "last_abandoned_date": "{}".format(last_abandoned_date),
#     "last_abandoned_product": "{}".format(last_abandoned_product),
#     "last_abandoned_url": "{}".format(last_abandoned_url)
#   }
# })
#   headers = {'Authorization' : 'Bearer {}'.format(token_key),
#              'content-type': 'application/json'}
#   response = requests.request("PATCH", url, headers=headers, data=payload).json()
#   result = json_normalize(response)
#   return result




# Copyright 2022 Thinh Vu @ GitHub
# See LICENSE for details.

import pandas as pd
import psycopg2 as ps
from configparser import ConfigParser
from sqlalchemy import create_engine

# DATABASE CONFIG
def db_config(section, db_cred_path):
    """
    Load credentials by a specific item name
    Args:
        item_name (:obj:`str`, required): name of the item that needs to load its value
        cred_path (:obj:`str`, required): Path to the .ini file where credentials are located.
    """
    parser = ConfigParser()
    parser.read(db_cred_path)
    db = {}
    if parser.has_section(section):
        params = parser.items(section)
        for param in params:
            db[param[0]] = param[1]
    else:
        raise Exception('Section {0} not found in the {1} file'.format(section, db_cred_path))
    return db

def db_query(query, section, db_cred_path):
  db_cred = db_config(section, db_cred_path)
  db_uri = 'postgresql://{}:{}@{}:{}/{}'.format(db_cred['user'], db_cred['password'], db_cred['host'], db_cred['port'], db_cred['database'])
  engine = create_engine(db_uri)
  try:
    query_result = pd.read_sql(query, engine)
    return query_result
  except (Exception, ps.DatabaseError) as error:
    print(error)




# Copyright 2022 Thinh Vu @ GitHub
# See LICENSE for details.
import yaml
import os
import requests
import shutil

def lmt_detect():
    if os.name == 'nt':
        lmt = '\\'
    else:
        lmt = '/'
    return lmt

ROOT_DIR = os.path.abspath(os.curdir)

def yaml_cred(item_name, cred_path):
    with open(cred_path) as file: 
        documents = yaml.full_load(file)
        for item, doc in documents.items():
            if item == item_name:
              secret_key = doc
              return secret_key

def get_google_font(font_family):
    lmt = lmt_detect()
    font_url = 'https://fonts.google.com/download?family={}'.format(font_family)
    response = requests.get(font_url)
    file_name = ROOT_DIR + lmt + '{}.zip'.format(font_family)
    with open(file_name, 'wb') as f:
        f.write(response.content)
    shutil.unpack_archive(file_name, lmt.join([ROOT_DIR, 'font', font_family]))





# I. Transcribe Youtube Videos
### !pip install git+https://github.com/openai/whisper.git -q
import os
from .util import lmt_detect, ROOT_DIR
import pytube
from pytube import YouTube
import whisper

def transcribe(link, model='medium'):
  lmt = lmt_detect()
  yt = YouTube(link)
  yt.streams.filter(only_audio=True)[0]\
          .download(output_path='') # save audio file to current directory
  file_path = lmt.join([ROOT_DIR, video_title(link)]) + '.mp4'
  os.system('whisper "{}" --model {}'.format(file_path, model))
  return file_path + '.txt'

def video_title(link):
  yt = YouTube(link)
  name = yt.title
  return name

## Free up memory from large model data
def free_memory():
    import gc
    import torch
    gc.collect()
    torch.cuda.empty_cache()



# Copyright 2022 Thinh Vu @ GitHub
# See LICENSE for details.
import pandas as pd
from oauth2client.service_account import ServiceAccountCredentials
import gspread
from df2gspread import df2gspread as d2g

def google_auth(json_cred_path):
    """
    Return a creds and gc to use Authenticate the Google service
    Args:
        json_cred_path (:obj:`str`, required): The path to the service account credential in json format.
    """
    scope = ['https://spreadsheets.google.com/feeds','https://www.googleapis.com/auth/drive']
    creds = ServiceAccountCredentials.from_json_keyfile_name(json_cred_path, scope)
    gc = gspread.authorize(creds)
    return creds, gc

def read_gsheet(sheet_id, sheet_name, gc):
    """
    Read a Google Sheets file to return a DataFrame. Given the fact that the target Google Sheets file has been shared to your Google Service Account.
    Args:
        sheet_id (:obj:`str`, required): Google Sheets ID. Ex: `YOUR_SHEET_ID` in the URI`https://docs.google.com/spreadsheets/d/YOUR_SHEET_ID/edit`
        sheet_name (:obj:`str`, required): Name of the worksheet
        gc (:obj, required): gc be assigned as to the result of google_auth() function.
    """
    wks = gc.open_by_key(sheet_id)
    worksheet = wks.worksheet(sheet_name)
    df = pd.DataFrame(worksheet.get_all_records())
    return df

def save_gsheet (dataframe, sheet_id, sheet_name, creds, row_names=False, clean=False, df_size=False, start_cell='A1'):
    """
    Export a DataFrame to Google Sheets file. Given the fact that the target Google Sheets file has been shared to your Google Service Account.
    Args:
        sheet_id (:obj, required): Target DataFrame to export.
        sheet_id (:obj:`str`, required): Google Sheets ID. Ex: `YOUR_SHEET_ID` in the URI`https://docs.google.com/spreadsheets/d/YOUR_SHEET_ID/edit`
        sheet_name (:obj:`str`, required): Name of the worksheet
        creds (:obj, required): creds be assigned as to the result of google_auth() function.
        rows_name (:obj:`boolean`, required): False as the default value.
        clean (:obj, required): False as the default value.
        df_size(:obj, required): False as the default value.
        start_cell (:obj, required): 'A1' as the default value.
    """
    d2g.upload(dataframe, sheet_id, sheet_name, credentials=creds, row_names=row_names, clean = clean, df_size = df_size, start_cell=start_cell)
    'https://docs.google.com/spreadsheets/d/' + sheet_id

def save_gspread (dataframe, sheet_id, sheet_name, gc):
    """
    Read a Google Sheets file to return a DataFrame. Given the fact that the target Google Sheets file has been shared to your Google Service Account.
    Args:
        sheet_id (:obj:`str`, required): Google Sheets ID. Ex: `YOUR_SHEET_ID` in the URI`https://docs.google.com/spreadsheets/d/YOUR_SHEET_ID/edit`
        sheet_name (:obj:`str`, required): Name of the worksheet
        gc (:obj, required): gc be assigned as to the result of google_auth() function.
    """
    try:
        wks = gc.open_by_key(sheet_id)
        wks.worksheet(sheet_name).update([dataframe.columns.values.tolist()] + dataframe.values.tolist())
        message = 'Data Frame has been exported to https://docs.google.com/spreadsheets/d/' + sheet_id
    except:
        wks = gc.open_by_key(sheet_id)
        rows, columns = dataframe.shape
        wks.add_worksheet(title=sheet_name, rows=rows, cols=columns)
        wks.worksheet(sheet_name).update([dataframe.columns.values.tolist()] + dataframe.values.tolist())
        message = 'Data Frame has been exported to https://docs.google.com/spreadsheets/d/' + sheet_id
    return message


import requests
import io
# import json
import os
import pandas as pd
from .util import lmt_detect

def appsflyer_report(token_key, app_id, report_type, start_date, end_date, api_version='v5'):
  """
  Export AppsFlyer Data to csv and save on your local machine.
  Args:
      token_key (:obj:`str`, required): AppsFlyer token key
      app_id (:obj:`str`, required): ID of your app. Ex: 'com.moneytap.vn.app','id1522245972'
      report_type (:obj: `str`, required): One of available AppsFlyer reports. Ex: 'installs_report', 'installs_report', 'in_app_events_report', 'organic_installs_report', 'organic_in_app_events_report', 'blocked_installs_report', 'blocked_in_app_events_report', 'detection', 'fraud-post-inapps'
      start_date (:obj: `str`, required): Using '%Y-%m-%d' format. Ex: '2022-09-25'
      end_date (:obj: `str`, required): Using '%Y-%m-%d' format. Ex: '2022-09-25'
      export_path (:obj: `str`, required): Local path to save the export file.
      api_version (:obj: `str`, optional): Leave it as is. See more: https://support.appsflyer.com/hc/en-us/articles/207034346-Aggregate-data-via-Pull-API-in-real-time#pull-api-for-developers
  """
  request_url = 'https://hq.appsflyer.com/export/{}/{}/{}'.format(app_id, report_type, api_version)
  params = {
      'api_token': '{}'.format(token_key),
      'from': '{}'.format(start_date),
      'to': '{}'.format(end_date)
    }
  res = requests.get(request_url, params=params)
  if res.status_code != 200:
    if res.status_code == 404:
      print('There is a problem with the request URL. Make sure that it is correct')
    else:
      print('There was a problem retrieving data: ', res.text)
  else:
    rawData = res.content
    df = pd.read_csv(io.StringIO(rawData.decode('utf-8')))
  return df


# Copyright 2022 Thinh Vu @ GitHub
# See LICENSE for details.
from datetime import datetime, date
from datetime import timedelta
import pandas as pd

def date_from_today(days_delta, format='%Y-%m-%d', backward = True):
    """
    Return a DateTime value using today's DateTime as the base.
    Args:
        days_delta (:obj:`int`, required): How many days to add or subtract from today's
        format (:obj:`str`, required): Datetime format of the output. Ex: "%Y-%m-%d". Concatnate the output string with 'T00:00:00Z' or 'T23:59:59Z' if you need.
        format (:obj:`boolean`, optional): True to get dates in the past, False to get the future ones
    """
    today_val = datetime.now()
    if backward == True:
        target = (today_val - timedelta(days_delta)).strftime(format)
    elif backward == False:
        target = (today_val + timedelta(days_delta)).strftime(format)
    else:
        pass
    return target


def date_start_end(periods, freq, format='%Y-%m-%d'):
    """
    Return a DateTime value using today's DateTime as the base.
    Args:
        format (:obj:`str`, required): Datetime format of the output. Ex: "%Y-%m-%d"
        periods (:obj:`int`, required): How many periods from today
        freq (:obj:`str`, required): datetime frequency code. Ex: 'MS' for Month Start and 'M' for Month End. See more at: 
    """
    today = datetime.now().strftime('%Y-%m-%d')
    target = pd.date_range(end=today, periods=periods, freq=freq)[0]\
                .strftime(format)
    return target


# generate week value
def weeknum_intel(datetime_now):
    """ Generate weeknum for the report, consider the lag time of ICT and PST timezone
    """
    if datetime_now.strftime('%a') == 'Mon':
        this_week = int(datetime_now.strftime('%U'))-1
        last_week = this_week - 1
    else:
        this_week = int(datetime_now.strftime('%U'))
        last_week = this_week - 1
    return this_week, last_week


def month_intel(datetime_now, end_date):
  "Input value should be assigned with datetime.now() value"
  if datetime_now.day == 1 and datetime_now.month < 2: # month = 1
      this_month = 'Dec'
      last_month = 'Nov'
      return this_month, last_month
  elif datetime_now.day > 1 and datetime_now.month < 2 : # month = 1
      this_month = 'Jan'
      last_month = 'Dec'
      return this_month, last_month
  if datetime_now.day == 1 and datetime_now.month == 2:
      this_month = 'Jan'
      last_month = 'Dec'
      return this_month, last_month
  elif datetime_now.day == 1 and datetime_now.month > 2:
      this_month = datetime(datetime_now.year, int(datetime.strptime(end_date, '%Y-%m-%d').strftime('%m')), 1).strftime('%b')
      last_month = datetime(datetime_now.year, int(datetime.strptime(end_date, '%Y-%m-%d').strftime('%m'))-1, 1).strftime('%b')
      return this_month, last_month
  else:
      this_month = datetime_now.strftime('%b')
      last_month = datetime(datetime_now.year, int(datetime.strptime(end_date, '%Y-%m-%d').strftime('%m'))-1, 1).strftime('%b')
      return this_month, last_month

def quarter_intel(datetime_now):
  """ Return this quarter, last quarter base on the datetime value of today"""
  if datetime_now.day == 1 and datetime_now.month in [1, 2, 3, 4]:
      this_quarter = 'Q1'
      last_quarter = 'Q4'
      return this_quarter, last_quarter
  elif datetime_now.day > 1 and datetime_now.month in [1, 2, 3]:
      this_quarter = 'Q1'
      last_quarter = 'Q4'
      return this_quarter, last_quarter
  elif datetime_now.day ==  1 and datetime_now.month in [7, 10]:
      this_quarter = f'Q{(datetime_now.month-1)//3}'
      last_quarter = f'Q{(datetime_now.month-1)//3-1}'
      return this_quarter, last_quarter
  else:
      this_quarter = f'Q{(datetime_now.month-1)//3+1}'
      last_quarter = f'Q{(datetime_now.month-1)//3}'
      return this_quarter, last_quarter